/*
 * HardwareInit.h
 *
 * Created: 01.01.2012 22:02:00
 *  Author: Philipp Bulling
 */ 


#ifndef HARDWAREINIT_H_
#define HARDWAREINIT_H_

void hardwareInit(void)
{
	// ====================================================================
	// Initialize LC-Display
	lcd_init();
	lcd_send(COMMAND, LCD_CLEAR);
	
	// ====================================================================
	// Initialize Rotary Switch:
	DDRC &= ~(1<<DDC1);	PORTC |= (1<<PC1);
	DDRC &= ~(1<<DDC2);	PORTC |= (1<<PC2);
	DDRC &= ~(1<<DDC3);	PORTC |= (1<<PC3);		
	
	// ====================================================================
	// Initialize Key at PIN B1:
	DDRB &= ~(1<<DDB1);	PORTB |= (1<<PB1);
	
	// ====================================================================
	// ADC Setup
	ADCSRA |= (1 << ADEN);						// ADC enable
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1);		// ADC clock 8 MHz / 64 = 125 kHz
	
	// ====================================================================
	// Initialize LED:
	DDRB |= (1<<DDB2);
	PORTB |= (1<<PB2);					// On by default
	
	// ====================================================================
	// Initialize Fast-PWM of Timer 2
	DDRB |= (1<<DDB3);					// PORT PB3 (OC2) output
	TCCR2 |= (1<<COM21);				// non inverting mode at pin OC2 (PB3):
										// clear output at threshold (=OC2) when incrementing, set output at TOP.
	TCCR2 |= (1<<WGM21);				// waveform generation mode (FAST PWM, 8 bit):
	TCCR2 |= (1<<WGM20);				// timer counts until 255
	TCCR2 |= (1<<CS20);					// PWM clock: 8 MHz = CPU
	TIMSK |= (1<<TOIE2);				// timer 2 overflow interrupt enabled
	TCNT2 = 0x00;						// start value for the timer counter
	
	// ====================================================================
	// Initialize Timer 1
	TCCR1B |= (1<<CS12);				// PWM clock: 8 MHz / 256 = 31250
	TIMSK |= (1<<TOIE1);				// timer 1 overflow interrupt enabled
	TCNT1 = 0xFFFF;						// start value for the timer counter
}

// ====================================================================
// The PWM counter counts up to 255, so the values of the sine are
// normalized from -1...1 to 0...255 (amplitude axis). The values below
// divide one period of the sine into 256 discrete values (time axis).
const uint8_t sinePeriod [256] = 
{
	0x7F, 0x82, 0x85, 0x88, 0x8C, 0x8F, 0x92, 0x95, 0x98, 0x9B, 0x9E, 0xA1, 0xA4, 0xA7, 
	0xAA, 0xAD, 0xB0, 0xB3, 0xB6, 0xB9, 0xBB, 0xBE, 0xC1, 0xC3, 0xC6, 0xC9, 0xCB, 0xCE, 
	0xD0, 0xD3, 0xD5, 0xD7, 0xD9, 0xDC, 0xDE, 0xE0, 0xE2, 0xE4, 0xE6, 0xE8, 0xE9, 0xEB, 
	0xED, 0xEE, 0xF0, 0xF1, 0xF2, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 
	0xFC, 0xFD, 0xFD, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFD, 
	0xFD, 0xFC, 0xFC, 0xFB, 0xFA, 0xFA, 0xF9, 0xF8, 0xF7, 0xF6, 0xF4, 0xF3, 0xF2, 0xF0, 
	0xEF, 0xED, 0xEC, 0xEA, 0xE8, 0xE7, 0xE5, 0xE3, 0xE1, 0xDF, 0xDD, 0xDB, 0xD8, 0xD6, 
	0xD4, 0xD1, 0xCF, 0xCC, 0xCA, 0xC7, 0xC5, 0xC2, 0xBF, 0xBD, 0xBA, 0xB7, 0xB4, 0xB1, 
	0xAF, 0xAC, 0xA9, 0xA6, 0xA3, 0xA0, 0x9D, 0x9A, 0x96, 0x93, 0x90, 0x8D, 0x8A, 0x87, 
	0x84, 0x81, 0x7D, 0x7A, 0x77, 0x74, 0x71, 0x6E, 0x6B, 0x68, 0x64, 0x61, 0x5E, 0x5B, 
	0x58, 0x55, 0x52, 0x4F, 0x4D, 0x4A, 0x47, 0x44, 0x41, 0x3F, 0x3C, 0x39, 0x37, 0x34, 
	0x32, 0x2F, 0x2D, 0x2A, 0x28, 0x26, 0x23, 0x21, 0x1F, 0x1D, 0x1B, 0x19, 0x17, 0x16, 
	0x14, 0x12, 0x11, 0x0F, 0x0E, 0x0C, 0x0B, 0x0A, 0x08, 0x07, 0x06, 0x05, 0x04, 0x04, 
	0x03, 0x02, 0x02, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x01, 0x01, 0x02, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0C, 
	0x0D, 0x0E, 0x10, 0x11, 0x13, 0x15, 0x16, 0x18, 0x1A, 0x1C, 0x1E, 0x20, 0x22, 0x25, 
	0x27, 0x29, 0x2B, 0x2E, 0x30, 0x33, 0x35, 0x38, 0x3B, 0x3D, 0x40, 0x43, 0x45, 0x48, 
	0x4B, 0x4E, 0x51, 0x54, 0x57, 0x5A, 0x5D, 0x60, 0x63, 0x66, 0x69, 0x6C, 0x6F, 0x72, 
	0x76, 0x79, 0x7C, 0x7F
};


#endif /* HARDWAREINIT_H_ */