/*
  ==========================================================
  	
	LCD Output for HD44780 compatible displays.
	
	lcd_write function edited by Philipp:
	-> not necessary to use lcd_set_cursor separately!
	Target: ATMEGA8
	
	Version: 1.0
	
  ==========================================================
*/

#define LCD_CLEAR		0x01	// Clear display: 		0b 0000 0001	
#define LCD_HOME		0x02	// Cursor home: 			0b 0000 0010
#define LCD_ON			0x0C	// Cursor invisible:	 	0b 0000 1100
#define LCD_OFF			0x08	// Display off: 			0b 0000 1000
#define POS_01			0x80	// Line 1 - Column 0		0b 1000 0000
#define POS_02			0xC0	// Line 2 - Column 0		0b 1100 0000

// definition of port pins - data 4-7 -> LCDPORT4-7
#define	LCDPORT			PORTD
#define	LCDDDR			DDRD
#define LCD_PIN_RS		1
#define LCD_PIN_E		3
#define LCD_PIN_D4		4
#define LCD_PIN_D5		5

#define	COMMAND			0
#define DATA			1

// General bit utilities
#define sbi(PORT, bit) 		(PORT|=(1<<bit)) 	// set bit in PORT		 	
#define cbi(PORT, bit) 		(PORT&=~(1<<bit))	// clear bit in PORT
#define tgl(PORT, bit) 		(PORT^=(1<<bit))	// toggle bit in PORT

// Function: Toggle enable pin
void toggle_enable_pin(void)
{	sbi(LCDPORT, LCD_PIN_E); cbi(LCDPORT, LCD_PIN_E);	
}

// Send Byte to LCD Controller
void lcd_send(unsigned char type, unsigned char c)
{	unsigned char sic_c;					// Backup for c
	
	// send high nibble
	sic_c = c;								// save original c
	sic_c &= ~0x0f;							// set bit 0-3 == 0
	if (type==DATA) 
		sic_c |= (1<<LCD_PIN_RS);			// Data: RS = 1
	LCDPORT = sic_c;						// send high nibble
	toggle_enable_pin();

	// send low nibble
	sic_c = c;								// save original c
	sic_c = sic_c<<4;						// exchange nibbles
	sic_c &= ~0x0f;							// set bit 0-3 == 0
	if (type==DATA)
		sic_c |= (1<<LCD_PIN_RS);			// Data: RS = 1
	LCDPORT = sic_c;						// send low nibble
	toggle_enable_pin();

	_delay_ms(5);							// Wait for LCD controller
}

// set cursor to line x and column y
void lcd_set_cursor(uint8_t x, uint8_t y)
{	uint8_t i;
	switch (x) 
  	{  	case 0: i=0x80+0x00+y; break;    // 1. line
    	case 1: i=0x80+0x40+y; break;    // 2. line
    	default: return;                 // invalid line
  	}
  	lcd_send(COMMAND, i);
}

// write string to LCD (uses lcd_set_cursor)
void lcd_write(uint8_t x, uint8_t y, char *data)
{	
	lcd_set_cursor(x, y); 
	while(*data) {lcd_send(DATA, *data); data++;}
}

// initialize LCD Controller 
void lcd_init()
{	// Set Port to Output
	LCDPORT = 0x00;	LCDDDR	= 0xFF;
	_delay_ms(50); // Wait for LCD

	// 4-bit Modus config
	sbi(LCDPORT, LCD_PIN_D5); cbi(LCDPORT, LCD_PIN_D4);

	// 4-Bit Modus start 
	sbi(PORTD, LCD_PIN_E); cbi(PORTD, LCD_PIN_E);
	_delay_ms(5); 

	// 2 Lines, 4-Bit Mode
	lcd_send(COMMAND, 0x28);
	lcd_send(COMMAND, LCD_OFF); lcd_send(COMMAND, LCD_CLEAR);
	lcd_send(COMMAND, 0x06);    lcd_send(COMMAND, LCD_ON);
}





